<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * Base class for service listing widgets
 *
 * @package   NINJA
 * @author    op5 AB
 * @license   GPL
 */
class tac_services_Base extends widget_Base {
	protected $duplicatable = true;
	protected $column_list = array(
			'acknowledged',
			'action_url',
			'active_checks_enabled',
			'description',
			'has_been_checked',
			'host.acknowledged',
			'host.active_checks_enabled',
			'host.has_been_checked',
			'host.name',
			'host.notifications_enabled',
			'host.pnpgraph_present',
			'host.scheduled_downtime_depth',
			'host.state',
			'last_check',
			'last_hard_state_change',
			'notes_url',
			'notifications_enabled',
			'plugin_output',
			'pnpgraph_present',
			'scheduled_downtime_depth',
			'state',
			);

	/**
	 * Return the default friendly name for the widget type
	 *
	 * default to the model name, but should be overridden by widgets.
	 */
	public function get_metadata() {
		return array_merge(parent::get_metadata(), array(
			'friendly_name' => 'Base class for service problem widgets',
			'instanceable' => false
		));
	}

	public function __construct($model) {
		parent::__construct($model);
		$service_group_list = Livestatus::instance()->getServicegroups(array('columns' => array('name', 'alias')));
		$this->services_groups = array(
			'all' => _('All')
		);
		if ($service_group_list !== false) {
			foreach($service_group_list as $group)
				$this->services_groups[$group['name']] = "{$group['alias']} ({$group['name']})";
		}

		$contact = ContactPool_Model::get_current_contact();
		if (!empty($contact)) {
			$this->cmd_ok = $contact->get_can_submit_commands();
		}
	}

	/**
	 * Create header links for status listing
	 */
	private function header_links(
			$type='host',
			$filter_object='all',
			$title=false,
			$method=false,
			$sort_field_db=false,
			$sort_field_str=false,
			$host_status=false,
			$service_status=false,
			$service_props=false,
			$host_props=false)
	{

		$type = trim($type);
		if (!is_string($filter_object))
			$filter_object = 'all';
		$filter_object = trim($filter_object);
		$title = trim($title);
		if (empty($type) || empty($title))  {
			return false;
		}
		$header = false;
		$lable_ascending = _('ascending');
		$lable_descending = _('descending');
		$lable_sort_by = _('Sort by');
		$lable_last = _('last');
		switch ($type) {
			case 'host':
				$header['title'] = $title;
				if (!empty($method) &&!empty($filter_object) && !empty($sort_field_db)) {
					$header['url_asc'] = Router::$controller.'/'.$method.'/'.$filter_object.'?hoststatustypes='.$host_status.'&sort_order='.nagstat::SORT_ASC.'&sort_field='.$sort_field_db;
					$header['alt_asc'] = $lable_sort_by.' '.$lable_last.' '.$sort_field_str.' ('.$lable_ascending.')';
					$header['img_asc'] = $this->img_sort_up;
					$header['url_desc'] = Router::$controller.'/'.$method.'/'.$filter_object.'?hoststatustypes='.$host_status.'&sort_order='.nagstat::SORT_DESC.'&sort_field='.$sort_field_db;
					$header['img_desc'] = $this->img_sort_down;
					$header['alt_desc'] = $lable_sort_by.' '.$sort_field_str.' ('.$lable_descending.')';
				}
				break;
			case 'service':
				$header['title'] = $title;
				if (!empty($method) &&!empty($filter_object) && !empty($sort_field_db)) {
					$header['url_asc'] = $this->model->name.'_'.$this->model->get_key().'_reorder(\''.$sort_field_db.'\', \''.nagstat::SORT_ASC.'\')';
					$header['img_asc'] = $this->img_sort_up;
					$header['alt_asc'] = $lable_sort_by.' '.$lable_last.' '.$sort_field_str.' ('.$lable_ascending.')';
					$header['url_desc'] = $this->model->name.'_'.$this->model->get_key().'_reorder(\''.$sort_field_db.'\', \''.nagstat::SORT_DESC.'\')';
					$header['img_desc'] = $this->img_sort_down;
					$header['alt_desc'] = $lable_sort_by.' '.$sort_field_str.' ('.$lable_descending.')';
				}
				break;
		}
		return $header;
	}

	/**
	 * List status details for hosts and services
	 *
	 * @param $name
	 * @param $servicestatustypes
	 * @param $hoststatustypes
	 * @param $sort_order
	 * @param $sort_field
	 * @param $group_type
	 */
	protected function service($name='all', $hoststatustypes=false, $servicestatustypes=false, $service_props=false, $sort_order='ASC', $sort_field='host.name', $group_type=false, $hostprops=false)
	{
		$query = '';
		$state = array();
		for ($i = 1; $i <= 4; $i++) {
			if ($servicestatustypes & (1 << $i))
				$state[] = 'state = '. $i;
		}
		if ($servicestatustypes & (1 << 6)) {
			$state[] = 'has_been_checked = 0';
		}
		if (empty($state))  {
			$query .= 'state = 99';
		} else {
			$query .= '('.implode(' or ', $state).')';
		}
		if ($service_props & nagstat::SERVICE_STATE_UNACKNOWLEDGED)
			$query .= ' and acknowledged = 0';
		if ($service_props & nagstat::SERVICE_STATE_ACKNOWLEDGED)
			$query .= ' and acknowledged = 1';
		if ($service_props & nagstat::SERVICE_CHECKS_ENABLED)
			$query .= ' and checks_enabled = 1';
		if ($service_props & nagstat::HOST_NO_SCHEDULED_DOWNTIME)
			$query .= ' and host.scheduled_downtime_depth = 0';
		if ($service_props & nagstat::SERVICE_HARD_STATE)
			$query .= ' and state_type = 1';
		if ($name && $name != 'all')
			$query .= ' and groups >= "'.$name.'"';

		$query = '[services] ' . $query;

		$mayi = op5MayI::instance();
		$result_set = ObjectPool_Model::get_by_query($query);
		$data = array();
		if ($mayi->run($result_set->mayi_resource().":read", array(), $messages)) {
			foreach( $result_set->it($this->column_list,array($sort_field .' '. $sort_order),100,0) as $elem ) {
				$data[] = $elem->export();
			}
		}
		else {
			$data["errors"] = $messages;
		}

		return $data;
	}

	public function options()
	{
		$options = parent::options();
		$opt = new option($this->model->name, 'service_box_height', 'Height (px)', 'input', array(), 0);
		$opt->should_render_js(false);
		$options[] = $opt;
		$options[] = new option($this->model->name, 'critical', 'Show critical', 'checkbox', array(), true);
		$options[] = new option($this->model->name, 'warning', 'Show warning', 'checkbox', array(), false);
		$options[] = new option($this->model->name, 'unknown', 'Show unknown', 'checkbox', array(), false);
		$options[] = new option($this->model->name, 'pending', 'Show pending', 'checkbox', array(), false);
		$options[] = new option($this->model->name, 'group', 'Service group', 'dropdown', array('options' => $this->services_groups), 'all');
		return $options;
	}

	protected function setup() {
		$this->arguments = $this->get_arguments();
		if (isset($_GET['sort_order'])) {
			$this->order = $_GET['sort_order'];
			$this->model->merge_settings(array('sort_order' => $this->order));
		} elseif (isset($this->arguments['sort_order'])) {
			$this->order = $this->arguments['sort_order'];
		} else {
			$this->order = 'ASC';
		}

		if (isset($_GET['sort_field'])) {
			$this->field = $_GET['sort_field'];
			$this->model->merge_settings(array('sort_field' => $this->field));
		} elseif (isset($this->arguments['sort_field'])) {
			$this->field = $this->arguments['sort_field'];
		} else {
			$this->field = 'host.name';
		}

		// Rewrite old value that is often broken
		if ($this->field == '-duration') {
			$this->field = 'last_hard_state_change';
		} else if ($this->field == 'h.host.name') {
			$this->field = 'host.name';
		}

		$this->state = 0;
		if ($this->arguments['critical'])
			$this->state |= nagstat::SERVICE_CRITICAL;
		if ($this->arguments['warning'])
			$this->state |= nagstat::SERVICE_WARNING;
		if ($this->arguments['unknown'])
			$this->state |= nagstat::SERVICE_UNKNOWN;
		if ($this->arguments['pending'])
			$this->state |= nagstat::SERVICE_PENDING;

		$this->img_sort_up = $this->add_path('icons/16x16/up.gif');
		$this->img_sort_down = $this->add_path('icons/16x16/down.gif');

		# assign specific header fields and values for current method
		$header_link_fields = array(
			array('title' => _('Host'), 'sort_field_db' => 'host.name', 'sort_field_str' => 'host name'),
			array('title' => _('Status'), 'sort_field_db' => 'state', 'sort_field_str' => 'service status'),
			array('title' => _('Service'), 'sort_field_db' => 'description', 'sort_field_str' => 'service name'),
			array('title' => _('Last Check'), 'sort_field_db' => 'last_check', 'sort_field_str' => 'last check time'),
			array('title' => _('Alert time'), 'sort_field_db' => 'last_hard_state_change', 'sort_field_str' => 'alert time'),
			array('title' => _('Status Information'))
		);

		# build header links array
		foreach ($header_link_fields as $fields) {
			if (sizeof($fields) > 1) {
				$this->header_link_strings[] = $this->header_links('service', $this->arguments['group'], $fields['title'], Router::$method, $fields['sort_field_db'], $fields['sort_field_str']);
			} else {
				$this->header_link_strings[] = $this->header_links('service', $this->arguments['group'], $fields['title']);
			}
		}

		$this->host_comments = $this->count_all_comments_by_host();
		$this->comments = $this->count_all_comments_by_service();

		$this->js = array('application/widgets/tac_services_common/js/tac_services_common');
	}

	/**
	*	Fetch comment counts for all objects that has comments
	*	Returned array will contain object name as key and count
	* 	as value for all objects with comments.
	*/
	private function count_all_comments_by_host()
	{
		$hosts_set = HostPool_Model::all()->reduce_by('comments', '', '!=');
		$data = array();
		foreach($hosts_set->it(array('comments')) as $object) {
			$count = count($object->get_comments());
			if($count) {
				$data[$object->get_key()] = $count;
			}
		}

		if( empty( $data ) ) {
			return false;
		}

		return $data;
	}

	/**
	*	Fetch comment counts for all objects that has comments
	*	Returned array will contain object name as key and count
	* 	as value for all objects with comments.
	*/
	private function count_all_comments_by_service()
	{
		$services_set = ServicePool_Model::all()->reduce_by('comments', '', '!=');
		$data = array();
		foreach($services_set->it(array('comments')) as $object) {
			$count = count($object->get_comments());
			if($count) {
				$data[$object->get_key()] = $count;
			}
		}

		if( empty( $data ) ) {
			return false;
		}

		return $data;
	}
}
